from numpy import ndarray
from numpy.core.multiarray import array as array
from rq_interfaces.msg import Volume
from visualization_msgs.msg import Marker
from .region_of_intrest import PyRegionOfIntrest
from sensor_msgs.msg import Image
from enum import IntEnum

import numpy as np
import ros2_numpy


class VOLUME_TYPES(IntEnum):
    UINT_16 = 0
    UINT_8 = 1


class PyVolume():
    def __init__(self, array: ndarray, roi: PyRegionOfIntrest, data_type: VOLUME_TYPES = ...):
        self.roi = roi
        self.array = array
        self.data_typ = data_type

    @staticmethod
    def get_data_type(volume_type: VOLUME_TYPES) -> np.dtype:
        if volume_type == VOLUME_TYPES.UINT_16:
            return np.uint16
        elif volume_type == VOLUME_TYPES.UINT_8:
            return np.uint8
        else:
            raise ValueError('Datatype is not implemented')
        
    @staticmethod
    def enum_to_numpify(volume_type: VOLUME_TYPES) -> str:
        if volume_type == VOLUME_TYPES.UINT_16:
            return 'mono16'
        elif volume_type == VOLUME_TYPES.UINT_8:
            return 'mono8'
        else:
            raise ValueError('Datatype is not implemented')
        
    @classmethod
    def dummy(cls):
        roi = PyRegionOfIntrest.dummy()
        array = np.random.randint(0, 255, size=roi.shape)
        data_type = VOLUME_TYPES.UINT_8
        return cls(array, roi, data_type)

    @classmethod
    def from_message(cls, msg: Volume):
        roi: Marker = msg.grid.region_of_intrest_stack.markers[0]
        center_points_mm = np.array([
            roi.pose.position.x,
            roi.pose.position.y,
            roi.pose.position.z])
            
        dimensions_mm = np.array([
            roi.scale.x,
            roi.scale.y,
            roi.scale.z])
        
        frame_id = roi.header.frame_id
        resolution_mm = np.array([
            msg.grid.resolution.x,
            msg.grid.resolution.y,
            msg.grid.resolution.z])
        
        py_roi = PyRegionOfIntrest(center_points_mm, dimensions_mm, frame_id, resolution_mm)
    
        data_typ = msg.datatype

        array = np.zeros(py_roi.shape, dtype=cls.get_data_type(data_typ))

        for i, slice in enumerate(msg.slices):
            array[:, :, i] = ros2_numpy.numpify(slice).reshape((py_roi.shape[0], py_roi.shape[1])).astype(cls.get_data_type(data_typ))

        return cls(array, py_roi, data_typ)
    
    def as_message(self) -> Volume:
        message = Volume()

        message.datatype = self.data_typ
        message.grid = self.roi.as_message()
        message.slices = list()

        for z in range(self.array.shape[2]):
            message.slices.append(
                ros2_numpy.msgify(Image, 
                                  self.array[:, :, z].astype(self.get_data_type(self.data_typ)), 
                                  self.enum_to_numpify(self.data_typ)))
        
        return message
    
    @property
    def shape(self):
        return self.array.shape
                                                         
        

        

